(function (global, factory) {
    typeof exports === 'object' && typeof module !== 'undefined' ? factory(exports, require('@angular/core'), require('rxjs/BehaviorSubject'), require('rxjs/observable/from'), require('rxjs/operators/map'), require('rxjs/operators/take'), require('rxjs/operators/filter'), require('rxjs/operators/tap')) :
    typeof define === 'function' && define.amd ? define('ngx-highlightjs', ['exports', '@angular/core', 'rxjs/BehaviorSubject', 'rxjs/observable/from', 'rxjs/operators/map', 'rxjs/operators/take', 'rxjs/operators/filter', 'rxjs/operators/tap'], factory) :
    (factory((global['ngx-highlightjs'] = {}),global.ng.core,global.Rx,global.Rx.Observable,global.Rx.Observable.prototype,global.Rx.Observable.prototype,global.Rx.Observable.prototype,global.Rx.Observable.prototype));
}(this, (function (exports,core,BehaviorSubject,from,map,take,filter,tap) { 'use strict';

    /**
     * @fileoverview added by tsickle
     * @suppress {checkTypes} checked by tsc
     */
    var /** @type {?} */ OPTIONS = new core.InjectionToken('OPTIONS');
    /**
     * @fileoverview added by tsickle
     * @suppress {checkTypes} checked by tsc
     */
    var HighlightService = (function () {
        /**
         * @param {?} options
         */
        function HighlightService(options) {
            this.options = {
                theme: 'github',
                path: 'assets/lib/hljs',
                auto: true
            };
            /**
             * hljs script on load event
             */
            this.ready$ = new BehaviorSubject.BehaviorSubject(false);
            this.options = Object.assign({}, this.options, options);
            if (typeof hljs !== 'undefined') {
                /** hljs is loaded by the user */
                hljs.configure(this.options.config);
                this.ready$.next(true);
            }
            else {
                /** Load hljs script and style locally */
                this.loadScript();
                this.loadTheme();
            }
        }
        /**
         * @return {?}
         */
        HighlightService.prototype.loadScript = function () {
            var _this = this;
            var /** @type {?} */ script = document.createElement('script');
            script.async = true;
            script.type = 'text/javascript';
            script.onload = function () {
                hljs.configure(_this.options.config);
                _this.ready$.next(true);
            };
            script.src = this.options.path + "/highlight.pack.js";
            document.head.appendChild(script);
        };
        /**
         * @return {?}
         */
        HighlightService.prototype.loadTheme = function () {
            var /** @type {?} */ style = document.createElement('link');
            style.rel = 'stylesheet';
            style.type = 'text/css';
            style.href = this.options.path + "/styles/" + this.options.theme + ".css";
            document.head.appendChild(style);
        };
        return HighlightService;
    }());
    HighlightService.decorators = [
        { type: core.Injectable },
    ];
    /** @nocollapse */
    HighlightService.ctorParameters = function () {
        return [
            { type: undefined, decorators: [{ type: core.Optional }, { type: core.Inject, args: [OPTIONS,] },] },
        ];
    };
    /**
     * @fileoverview added by tsickle
     * @suppress {checkTypes} checked by tsc
     */
    /**
     * There are 2 ways to higlight a code
     *  1 - using the [code] input (default) <code highlight [code]="yourCode"></code>
     *  2 - using element text content <code> {{yourCode}} </code>
     */
    var HighlightDirective = (function () {
        /**
         * @param {?} el
         * @param {?} renderer
         * @param {?} hl
         */
        function HighlightDirective(el, renderer, hl) {
            this.renderer = renderer;
            this.hl = hl;
            this.highlighted = new core.EventEmitter();
            this.el = el.nativeElement;
        }
        Object.defineProperty(HighlightDirective.prototype, "setCode", {
            /**
             * @param {?} code
             * @return {?}
             */
            set: function (code) {
                var _this = this;
                this.code = code;
                this.hl.ready$.pipe(filter.filter(function (ready) { return ready; }), take.take(1), tap.tap(function () { return _this.highlightElement(_this.el, code); })).subscribe();
            },
            enumerable: true,
            configurable: true
        });
        /**
         * @return {?}
         */
        HighlightDirective.prototype.ngOnInit = function () {
            var _this = this;
            /** Acitvate MutationObserver if `auto` option is true and `[code]` input is not used
                 * This will highlight using the text content */
            if (!this.code && this.hl.options.auto) {
                this.hl.ready$.pipe(filter.filter(function (ready) { return ready; }), take.take(1), tap.tap(function () {
                    _this.highlightTextContent();
                    /** Highlight when text content changes */
                    _this.domObs = new MutationObserver(function () { return _this.highlightTextContent(); });
                    _this.domObs.observe(_this.el, { childList: true, subtree: true });
                })).subscribe();
            }
        };
        /**
         * Highlight using element text content
         * @return {?}
         */
        HighlightDirective.prototype.highlightTextContent = function () {
            if (!this.highlight) {
                if (this.el.tagName.toLowerCase() === 'code') {
                    this.highlightElement(this.el, this.el.innerText.trim());
                }
                else {
                    console.warn("[HighlightDirective]: Use 'highlight' on <code> element only");
                }
            }
            else if (this.highlight === 'all') {
                this.highlightChildren(this.el, 'pre code');
            }
            else {
                this.highlightChildren(this.el, this.highlight);
            }
        };
        /**
         * Highlight a code block
         * @param {?} el
         * @param {?} code
         * @return {?}
         */
        HighlightDirective.prototype.highlightElement = function (el, code) {
            var /** @type {?} */ res = hljs.highlightAuto(code, this.language);
            if (res.value !== el.innerHTML) {
                this.renderer.addClass(el, 'hljs');
                this.renderer.setProperty(el, 'innerHTML', res.value);
                this.highlighted.emit(res);
            }
        };
        /**
         * Highlight multiple code blocks
         * @param {?} el
         * @param {?} selector
         * @return {?}
         */
        HighlightDirective.prototype.highlightChildren = function (el, selector) {
            var _this = this;
            var /** @type {?} */ codeElements = el.querySelectorAll(selector);
            /** highlight children with the same selector */
            from.from(codeElements).pipe(filter.filter(function (code) { return code.childNodes.length === 1 && code.childNodes[0].nodeName === '#text'; }), map.map(function (codeElement) { return _this.highlightElement(codeElement, codeElement.innerText.trim()); }), take.take(1)).subscribe();
        };
        /**
         * @return {?}
         */
        HighlightDirective.prototype.ngOnDestroy = function () {
            /** Disconnect MutationObserver */
            if (!this.code && this.hl.options.auto) {
                this.domObs.disconnect();
            }
        };
        return HighlightDirective;
    }());
    HighlightDirective.decorators = [
        { type: core.Directive, args: [{
                    selector: '[highlight]'
                },] },
    ];
    /** @nocollapse */
    HighlightDirective.ctorParameters = function () {
        return [
            { type: core.ElementRef, },
            { type: core.Renderer2, },
            { type: HighlightService, },
        ];
    };
    HighlightDirective.propDecorators = {
        "highlight": [{ type: core.Input },],
        "language": [{ type: core.Input },],
        "setCode": [{ type: core.Input, args: ['code',] },],
        "highlighted": [{ type: core.Output },],
    };
    /**
     * @fileoverview added by tsickle
     * @suppress {checkTypes} checked by tsc
     */
    /**
     * @param {?} options
     * @return {?}
     */
    function HighlightFactory(options) {
        return new HighlightService(options);
    }
    var HighlightModule = (function () {
        function HighlightModule() {
        }
        /**
         * @param {?=} options
         * @return {?}
         */
        HighlightModule.forRoot = function (options) {
            return {
                ngModule: HighlightModule,
                providers: [
                    { provide: OPTIONS, useValue: options },
                    {
                        provide: HighlightService,
                        useFactory: HighlightFactory,
                        deps: [OPTIONS]
                    }
                ]
            };
        };
        return HighlightModule;
    }());
    HighlightModule.decorators = [
        { type: core.NgModule, args: [{
                    declarations: [HighlightDirective],
                    exports: [HighlightDirective]
                },] },
    ];

    exports.HighlightModule = HighlightModule;
    exports.HighlightDirective = HighlightDirective;

    Object.defineProperty(exports, '__esModule', { value: true });

})));
//# sourceMappingURL=ngx-highlightjs.umd.js.map
